package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.GlobalVar;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.app.customer.service.api.service.InnerApiService;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.GicClerkDTO;
import com.gic.haoban.manage.api.dto.GicMessageDTO;
import com.gic.haoban.manage.api.dto.QywxCallBackDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.StaffDepartmentRelatedDTO;
import com.gic.haoban.manage.api.dto.WxApplicationDTO;
import com.gic.haoban.manage.api.enums.GicDataType;
import com.gic.haoban.manage.api.enums.GicEditType;
import com.gic.haoban.manage.api.enums.WxEditType;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.MessageApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.service.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.thirdparty.api.dto.PicUploadResDTO;
import com.gic.thirdparty.api.service.QQCloudPicService;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;

import java.io.InputStream;
import java.net.URL;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 接收企业微信或者gic推送消息进行处理
 * Created by tgs on 2020/2/20.
 */
@Service("messageApiService")
public class MessageApiServiceImpl implements MessageApiService {
    private static final Logger log = LogManager.getLogger(MessageApiServiceImpl.class);

    @Autowired
    private DepartmentService departmentService;
    @Autowired
    private WxEnterpriseRelatedService wxEnterpriseRelatedService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private WxEnterpriseService enterpriseService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private StaffDepartmentRelatedService staffDepartmentRelatedService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private WxApplicationService wxApplicationService;
    @Autowired
    private Config config;
    @Autowired
    private QywxDepartmentApiService qywxDepartmentApiService;
    
    @Autowired
	private QQCloudPicService qqCloudPicService;
    
    @Autowired
    private InnerApiService innerApiService;
    
    private final String CONTACT_APP = "contactSuiteId";
    
    @Autowired
    private StoreService storeService;

    @Override
    public void gicMessageReceive(GicMessageDTO gicMessageDTO) {
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            log.info("发送消息到dealGicMessageMq，{}", JSON.toJSONString(gicMessageDTO));
            clientInstance.sendMessage("dealGicMessageMq", JSON.toJSONString(gicMessageDTO));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void wxMessageReceive(QywxCallBackDTO qywxCallBackDTO) {
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            if(config.getSuiteId().equals(qywxCallBackDTO.getSuiteId()) || CONTACT_APP.equals(qywxCallBackDTO.getSuiteId())){
                log.info("发送消息到dealWxMessageMq，{}", JSON.toJSONString(qywxCallBackDTO));
                clientInstance.sendMessage("dealWxMessageMq", JSON.toJSONString(qywxCallBackDTO));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void dealGicMessage(String param) {
        if (true) {
            return;
        }
        GicMessageDTO dto = JSON.parseObject(param, GicMessageDTO.class);
        TabHaobanWxEnterpriseRelated enterprise =
                this.wxEnterpriseRelatedService.getEnterpriseByGicEnterpriseId(dto.getEnterpriseId());
        if(enterprise != null){
            DepartmentDTO departmentDTO = new DepartmentDTO();
            departmentDTO.setDepartmentName(dto.getName());
            departmentDTO.setIsStore(dto.getType() != GicDataType.STORE.getCode() ? 0 : 1);
            departmentDTO.setRelatedId(dto.getId());
            log.info("【部门同步】dto={}",JSON.toJSONString(dto));
            DepartmentDTO parentDepartment = this.getParentDepartment(dto.getParentChain(), dto.getType(), enterprise.getWxEnterpriseId());
            if(parentDepartment == null && (dto.getChangeType() == GicEditType.ADD.getCode() || dto.getChangeType() == GicEditType.UPDATE.getCode())){
            	if(dto.getChangeType() == GicEditType.UPDATE.getCode()){
            		String departmentId = dto.getId();
            		DepartmentDTO exsit = departmentApiService.selectByRelatedId(departmentId);
            		if(exsit == null){
            			log.info("【部门同步】父部门为空");
            			return;  
            		}else{
            			parentDepartment = departmentApiService.selectById(exsit.getParentDepartmentId());
            		}
            	}
            	if(parentDepartment == null){
    				log.info("【部门同步】父部门为空");
                	return;  
    			}
            }
            if(parentDepartment != null){
            	 departmentDTO.setParentDepartmentId(parentDepartment.getDepartmentId());
                 departmentDTO.setChainId(parentDepartment.getChainId()+ Constant.ID_SEPARATOR + parentDepartment.getDepartmentId());
                 departmentDTO.setChainName(parentDepartment.getChainName() + Constant.NAME_SEPARATOR + parentDepartment.getChainName());
                 departmentDTO.setLevel(parentDepartment.getLevel() + 1);
            }
            departmentDTO.setWxEnterpriseId(enterprise.getWxEnterpriseId());
            if(dto.getChangeType() == GicEditType.ADD.getCode()){
            	Integer isStore = departmentDTO.getIsStore();
                if(isStore == 1){
                	//不是线上门店 不新增
                	StoreDTO store = storeService.getStore(departmentDTO.getRelatedId());
                	if(store == null || store.getStatus() != 2){
                		return;
                	}
                }
                ServiceResponse response = this.departmentApiService.add(departmentDTO);
                GicClerkDTO gicClerkDTO = dto.getGicClerkDTO();
                if(gicClerkDTO != null){
                	gicClerkDTO.setPostionName("店长");
                	staffApiService.addGicClerk(gicClerkDTO);
                }
                log.info("添加结果：{}", JSON.toJSONString(response));
            }else if(dto.getChangeType() == GicEditType.UPDATE.getCode()){
                TabHaobanDepartment department = this.departmentService.selectByRelatedId(dto.getId());
                if(department == null){
                	Integer isStore = departmentDTO.getIsStore();
                    if(isStore == 1){
                    	//不是线上门店 不新增
                    	StoreDTO store = storeService.getStore(departmentDTO.getRelatedId());
                    	if(store == null || store.getStatus() != 2){
                    		return;
                    	}
                    }
                    ServiceResponse response = this.departmentApiService.add(departmentDTO);
                    log.info("添加结果：{}", JSON.toJSONString(response));
                }else {
                    departmentDTO.setDepartmentId(department.getDepartmentId());
                    this.departmentApiService.edit(departmentDTO);
                    log.info("gic同步更新完成");
                }
            }else {
                TabHaobanDepartment department = this.departmentService.selectByRelatedId(dto.getId());
                if(department != null){
                    this.departmentApiService.gicDel(department.getDepartmentId());
                    log.info("gic同步删除完成");
                }else {
                    log.info("gic同步删除不存在");
                }
            }
        }

    }

    @Override
    public void dealWxMessage(String param) {
        QywxCallBackDTO dto = JSON.parseObject(param, QywxCallBackDTO.class);
        log.info("【处理微信回调】dto={}",JSON.toJSONString(param));
        if(dto != null){
        	String corpId = dto.getAuthCorpId();
        	TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(corpId);
        	if(enterprise != null){
    			 if(dto.getChangeType().endsWith("party")){
    		            this.dealDepartment(dto);
    		        }else {
    		            this.dealUser(dto);
    		        }
        	}
        }
    }

    private void dealDepartment(QywxCallBackDTO qywxCallBackDTO){
        DepartmentDTO departmentDTO = new DepartmentDTO();
        departmentDTO.setDepartmentName(qywxCallBackDTO.getDepartmentName());
        departmentDTO.setIsStore(0);
        //departmentDTO.setRelatedId(qywxCallBackDTO.getId()+"");
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        TabHaobanDepartment parentDepartment = new TabHaobanDepartment();
        if(qywxCallBackDTO.getParentId() != null){
        	parentDepartment = this.departmentService.getByWxId(qywxCallBackDTO.getParentId() + "", enterprise.getWxEnterpriseId());
        }else{
        	TabHaobanDepartment oldDepartment = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
        	if(oldDepartment != null){
        		parentDepartment = departmentService.selectById(oldDepartment.getParentDepartmentId());
        	}
        }
        if(!qywxCallBackDTO.getChangeType().equals(WxEditType.DELETEDEPART.getCode())){
        	 if(parentDepartment == null){
            	log.info("微信同步父部门不存在");
                return;
            }
        	departmentDTO.setParentDepartmentId(parentDepartment.getDepartmentId());
            departmentDTO.setChainId(parentDepartment.getChainId()+ Constant.ID_SEPARATOR + parentDepartment.getDepartmentId());
            departmentDTO.setChainName(parentDepartment.getChainName() + Constant.NAME_SEPARATOR + parentDepartment.getChainName());
            departmentDTO.setLevel(parentDepartment.getLevel() + 1);
        }
        departmentDTO.setWxEnterpriseId(enterprise.getWxEnterpriseId());
        departmentDTO.setWxDepartmentId(qywxCallBackDTO.getId()+"");
        if(qywxCallBackDTO.getParentId() != null){
        	List<com.gic.wechat.api.dto.qywx.DepartmentDTO> list = this.qywxDepartmentApiService.listDepartment(qywxCallBackDTO.getAuthCorpId(), qywxCallBackDTO.getSuiteId(), qywxCallBackDTO.getParentId());
        	if(list != null){
        		for (com.gic.wechat.api.dto.qywx.DepartmentDTO departmentDTO2 : list) {
    				if(departmentDTO2.getId().equals(departmentDTO.getDepartmentId())){
    					departmentDTO.setSort(qywxCallBackDTO.getOrder());
    				}else{
    					TabHaobanDepartment tab = this.departmentService.getByWxId(departmentDTO2.getId() + "", enterprise.getWxEnterpriseId());
    					if(tab != null){
    						tab.setSort(departmentDTO2.getOrder());
    						this.departmentService.edit(EntityUtil.changeEntityByJSON(DepartmentDTO.class, tab));
    					}
    				}
    			}
        	}
        }
        if(qywxCallBackDTO.getChangeType().equals(WxEditType.ADDDEPART.getCode())){
            String response = this.departmentService.add(departmentDTO);
            log.info("微信同步新增部门：{}", JSON.toJSONString(response));
        }else if(qywxCallBackDTO.getChangeType().equals(WxEditType.UPDATEDEPART.getCode())){
            TabHaobanDepartment department = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if(department == null){
                String response = this.departmentService.add(departmentDTO);
                log.info("微信同步新增部门：{}", JSON.toJSONString(response));
            }else {
            	departmentDTO.setIsStore(department.getIsStore());
                departmentDTO.setDepartmentId(department.getDepartmentId());
                this.departmentService.edit(departmentDTO);
                log.info("企业微信更新同步完成");
            }
        }else {
            TabHaobanDepartment department = this.departmentService.getByWxId(qywxCallBackDTO.getId() + "", enterprise.getWxEnterpriseId());
            if(department == null){
                log.info("微信同步删除部门不存在");
            }else {
            	if(StringUtils.isBlank(department.getRelatedId())){
            		this.departmentService.del(department.getDepartmentId());
            	}else{
            		this.departmentApiService.wxCallBackrecycle(department.getDepartmentId());
            	}
                log.info("企业微信删除同步完成");
            }
        }
    }

    private void dealUser(QywxCallBackDTO qywxCallBackDTO){
        TabHaobanStaff staff = new TabHaobanStaff();
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        if(enterprise != null){
            staff.setWxEnterpriseId(enterprise.getWxEnterpriseId());
            //staff.setActiveFlag(0);
            staff.setPhoneNumber(qywxCallBackDTO.getMobile());
            staff.setNickName(qywxCallBackDTO.getAlias());
            staff.setSex(qywxCallBackDTO.getGender());
            staff.setStaffName(qywxCallBackDTO.getUserName());
            staff.setWxUserId(qywxCallBackDTO.getUserid());
            staff.setPostion(qywxCallBackDTO.getPosition());
            if(StringUtils.isNotBlank(qywxCallBackDTO.getAvatar())){
            	staff.setHeadImg(changeHeaderImageUrl(qywxCallBackDTO.getAvatar()));
            }
            staff.setNationCode("86");
            //激活状态
            if(qywxCallBackDTO.getStatus() != null && qywxCallBackDTO.getStatus() == 1){
            	staff.setActiveFlag(1);
            }
            if(qywxCallBackDTO.getChangeType().equals(WxEditType.ADDUSER.getCode())){
            	if(staff.getActiveFlag() == null){
            		staff.setActiveFlag(0);
            	}
                this.addUser(qywxCallBackDTO, staff);
            } else if(qywxCallBackDTO.getChangeType().equals(WxEditType.UPDATEUSER.getCode())){
                TabHaobanStaff oldStaff = this.staffService.selectByUserIdAndEnterpriseId(qywxCallBackDTO.getUserid(),enterprise.getWxEnterpriseId());
                if(oldStaff != null){
                    staff.setWxUserId(qywxCallBackDTO.getNewUserid());
                    staff.setStaffId(oldStaff.getStaffId());
                    log.info("【微信回调更新】staff={}",JSON.toJSONString(staff));
                    if(StringUtils.isBlank(staff.getPhoneNumber())){
                    	staff.setPhoneNumber(oldStaff.getPhoneNumber());
                    }
                    if(StringUtils.isBlank(staff.getNationCode())){
                    	staff.setNationCode(oldStaff.getNationCode());
                    }
                    if(StringUtils.isBlank(staff.getStaffName())){
                    	staff.setStaffName(oldStaff.getStaffName());
                    }
                    if(StringUtils.isBlank(staff.getPostion())){
                    	staff.setPostion(oldStaff.getPostion());
                    }
                    if(staff.getSex() == null){
                    	staff.setSex(oldStaff.getSex());
                    }
                    if(staff.getActiveFlag() == null){
                    	staff.setActiveFlag(oldStaff.getActiveFlag());
                    }
                    if(StringUtils.isBlank(staff.getHeadImg())){
                    	staff.setHeadImg(oldStaff.getHeadImg());
                    }
//                    this.staffService.updateByPrimaryKey(staff);
//                    this.staffDepartmentRelatedService.delByUserid(qywxCallBackDTO.getUserid());
//                    this.addUserDepart(qywxCallBackDTO, oldStaff.getStaffId(), enterprise.getWxEnterpriseId());
                    String[] departArr = qywxCallBackDTO.getDepartment();
                    String departmentIds = "";
                    //部门修改了
                    if(departArr != null){
                        for(String s : departArr){
                        	TabHaobanDepartment department = this.departmentService.getByWxId(s, enterprise.getWxEnterpriseId());
                            if(department == null){
                                log.info("部门不存在");
                                continue;
                            }
                            departmentIds += department.getDepartmentId() + ",";
                        }
                        departmentIds = departmentIds.substring(0,departmentIds.length() - 1);
                    }else{
                    	//部门没有修改
                    	List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
                    	for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : list) {
                    		 departmentIds += tabHaobanStaffDepartmentRelated.getDepartmentId() + ",";
						}
                    	departmentIds = departmentIds.substring(0,departmentIds.length() - 1);
                    }
                    StaffDTO staffDTO = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
                    staffDTO.setWeixinPush(true);
                    staffApiService.staffEdit(staffDTO, departmentIds);
                    
                }else {
                	if(staff.getActiveFlag() == null){
                		staff.setActiveFlag(0);
                	}
                    this.addUser(qywxCallBackDTO, staff);
                }
            } else {
                TabHaobanStaff oldStaff = this.staffService.selectByUserIdAndEnterpriseId(qywxCallBackDTO.getUserid(),enterprise.getWxEnterpriseId());
                //List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
                if(oldStaff != null){
                	List<TabHaobanStaffDepartmentRelated> relatedList = staffDepartmentRelatedService.listStaffDepartmentByStaffId(oldStaff.getStaffId());
                	TabHaobanStaffDepartmentRelated staffRelated = relatedList.get(0);
    				if(StringUtils.isNotBlank(staffRelated.getClerkCode())){
    					staffApiService.cleanGicClerk(staffRelated.getClerkCode(), staffRelated.getDepartmentId());
    				}
    				innerApiService.delCardByStaffId(oldStaff.getStaffId());
    				this.staffDepartmentRelatedService.del(EntityUtil.changeEntityByJSON(StaffDepartmentRelatedDTO.class, staffRelated));
    				this.staffService.delByuserid(oldStaff.getStaffId());
                }else {
                    log.info("微信同步删除的门店不存在，{}", qywxCallBackDTO.getUserid());
                }
            }
        }else {
            log.info("授权企业不存在，{}", qywxCallBackDTO.getAuthCorpId());
        }
    }


    private void addUser(QywxCallBackDTO qywxCallBackDTO, TabHaobanStaff staff){
    	TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
    	StaffDTO staffDTO = staffApiService.selectByUserIdAndEnterpriseId(staff.getWxUserId(),enterprise.getWxEnterpriseId());
    	String[] departArr = qywxCallBackDTO.getDepartment();
    	String departmentIds = "";
    	for(String s : departArr){
        	TabHaobanDepartment department = this.departmentService.getByWxId(s, enterprise.getWxEnterpriseId());
            if(department == null){
                log.info("部门不存在");
                continue;
            }
            departmentIds += department.getDepartmentId() + ",";
        }
        departmentIds = departmentIds.substring(0,departmentIds.length() - 1);
    	if(staffDTO == null){
    		staffApiService.wxGetAdd(staff.getWxUserId(), enterprise.getWxEnterpriseId());
    	}else{
    		List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
        	for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : list) {
        		if(!departmentIds.contains(tabHaobanStaffDepartmentRelated.getDepartmentId())){
        			departmentIds += tabHaobanStaffDepartmentRelated.getDepartmentId() + ",";
        		}
        		if(departmentIds.endsWith(",")){
        			departmentIds = departmentIds.substring(0,departmentIds.length() - 1);
        		}
			}
        	StaffDTO staffUpdate = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
        	staffUpdate.setWeixinPush(true);
        	staffApiService.staffEdit(staffUpdate, departmentIds);
    	}
//        String add = this.staffService.add(staff);
//        log.info("新增用户返回结果：{}", add);
//        this.addUserDepart(qywxCallBackDTO, add, staff.getWxEnterpriseId());
    }

    private void addUserDepart(QywxCallBackDTO qywxCallBackDTO, String staffId, String wxEnterpriseId){
        String[] departArr = qywxCallBackDTO.getDepartment();
        if(departArr != null){
            for(String s : departArr){
                TabHaobanDepartment department = this.departmentService.getByWxId(s, wxEnterpriseId);
                if(department == null){
                    log.info("部门不存在");
                    continue;
                }
                StaffDepartmentRelatedDTO related = new StaffDepartmentRelatedDTO();
                related.setWxEnterpriseId(wxEnterpriseId);
                related.setWxUserId(qywxCallBackDTO.getUserid());
                related.setClerkCode(qywxCallBackDTO.getUserid());
                related.setDepartmentId(department.getDepartmentId());
                related.setPhoneNumber(qywxCallBackDTO.getMobile());
                //related.setStaffDepartmentRelatedId(staffId);
                related.setStaffId(staffId);
                related.setStatusFlag(1);
                String add1 = this.staffDepartmentRelatedService.add(related);
                log.info("新增用户部门关联关系：{}", add1);
            }
        }
    }
    
    private String changeHeaderImageUrl(String headImgUrl) {

		try {
			InputStream in = new URL(headImgUrl).openStream();
			byte[] data = IOUtils.toByteArray(in);
			PicUploadResDTO uploadPic = qqCloudPicService.uploadPic(GlobalVar.ctxPropertiesMap.get(GlobalInfo.QQPIC_KEY_ENTERPRISE), data);
			log.info("腾讯云万象优图返回" + JSON.toJSONString(uploadPic));
			return uploadPic.downloadUrl;
		}catch (Exception e){
			log.info("上传腾讯云万象优图返回失败了：", e);
			return headImgUrl;
		}
	}
    /**
     * 通过关联的父级ID链查询父级部门
     * @param parentChain
     * @return
     */
    private DepartmentDTO getParentDepartment(String parentChain, int dataType, String wxEnterpriseId){
        if(StringUtils.isNotBlank(parentChain)){
            String[] arr = parentChain.split("_");
            int len = arr.length;
            int startIndex = len - 2;
            if(dataType == GicDataType.STORE.getCode() || dataType == GicDataType.DEPARTMENT.getCode()){
                startIndex = len - 1;
            }
            for(int i = startIndex ; i>=0 && i< len; i--){
                DepartmentDTO department = this.departmentService.getByRelatedIdNotInRecycle(arr[i]);
                log.info("当前节点id：{}, {}", arr[i], JSON.toJSONString(department));
                if(department != null){
                    return department;
                }
            }
        }
        //DepartmentDTO departmentDTO = this.departmentService.getRootByEnterpriseId(wxEnterpriseId);
        return null;
    }

    public static void main(String[] args){
        String parentChain = "1_";
        String[] arr = parentChain.split("_");
        int len = arr.length;
        int startIndex = len - 1;
        System.out.println(len);
        for(int i = startIndex ; i>=0 && i< len; i--){
            System.out.println(i);
        }
    }
}
